using System;
using System.Data;
using System.Configuration;
using System.Collections;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using Traysoft.AddTapi;

namespace Dial
{
    public partial class _Default : System.Web.UI.Page
    {
        protected void Page_Load(object sender, EventArgs e)
        {
            if (!IsPostBack) FillLinesDropDown();
        }

        protected void ButtonDial_Click(object sender, EventArgs e)
        {
            LabelError.Text = null;
            try
            {
                // Get TapiLine object for selected line
                TapiLine line = GetLineByName(DropDownListLine.SelectedItem.Text);
                if (line == null)
                {
                    LabelError.Text = "Please select a line for outgoing call";
                    return;
                }
                // Open the line for outgoing calls if it wasn't open before
                if (!line.IsOpen) line.Open(false, null, TapiMediaMode.InteractiveVoice);
                // Dial the number
                string number = TextBoxNumber.Text.Trim();
                if (String.IsNullOrEmpty(number))
                {
                    LabelError.Text = "Please enter a number to dial";
                    return;
                }
                line.Dial(number, false, TapiMediaMode.InteractiveVoice);
            }
            catch (TapiException exc)
            {
                LabelError.Text = exc.Message;
            }
        }

        protected void ButtonHangup_Click(object sender, EventArgs e)
        {
            LabelError.Text = null;
            try
            {
                // Get TapiLine object for selected line
                TapiLine line = GetLineByName(DropDownListLine.SelectedItem.Text);
                if (line == null)
                {
                    LabelError.Text = "Please select a line";
                    return;
                }
                TapiCall activeCall = null;
                foreach(TapiCall call in line.Calls) {
                    if (call.State != TapiCallState.Idle && call.State != TapiCallState.Disconnected) activeCall = call;
                }
                if (activeCall != null) activeCall.Disconnect();
                else
                {
                    LabelError.Text = "No active calls on line " + line.Name;
                    return;
                }                
            }
            catch (TapiException exc)
            {
                LabelError.Text = exc.Message;
            }
        }

        void FillLinesDropDown()
        {
            foreach (TapiLine line in TapiApp.Lines)
                DropDownListLine.Items.Add(line.Name);            
        }

        TapiLine GetLineByName(string name)
        {
            foreach (TapiLine line in TapiApp.Lines)
                if (line.Name == name) return line;
            return null;
        }
    }
}
