Option Explicit On

Imports Traysoft.AddTapi

Public Class Form1
    Delegate Sub AddListItem(ByVal item As String)
    Dim m_addToLogDelegate As AddListItem

    Private Sub Form1_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        m_addToLogDelegate = New AddListItem(AddressOf AddToLog)
        Try
            ' Set the serial number if you purchased AddTapi.NET
            'TapiApp.SerialNumber = "ABCD-EFGH-IJKL"
            ' Initialize AddTapi
            TapiApp.Initialize("AutomatedCalls")
            ' Subscribe to events from AddTapi
            AddHandler TapiApp.TapiError, AddressOf OnTapiError
            AddHandler TapiApp.LineAdded, AddressOf OnLineAdded
            AddHandler TapiApp.LineClosed, AddressOf OnLineClosed
            AddHandler TapiApp.LineRemoved, AddressOf OnLineRemoved
            AddHandler TapiApp.CallDisconnected, AddressOf OnCallDisconnected
            ' Fill out voice combobox with a list of available text-to-speech voices
            For Each voice As TapiVoice In TapiApp.Voices
                comboBoxVoice.Items.Add(voice)
            Next
            ' Show line names in the combobox
            UpdateLinesCombobox()
        Catch exc As TapiException
            MessageBox.Show(exc.Message, "TapiException!", MessageBoxButtons.OK, MessageBoxIcon.Error)
            Me.Close()
        Catch exc As Exception
            MessageBox.Show(exc.Message, "Exception!", MessageBoxButtons.OK, MessageBoxIcon.Error)
            Me.Close()
        End Try
    End Sub

    Private Sub Form1_FormClosed(ByVal sender As System.Object, ByVal e As System.Windows.Forms.FormClosedEventArgs) Handles MyBase.FormClosed
        Try
            TapiApp.Shutdown()
        Catch exc As TapiException
            MessageBox.Show(exc.Message, "TapiException!", MessageBoxButtons.OK, MessageBoxIcon.Error)
        Catch exc As Exception
            MessageBox.Show(exc.Message, "Exception!", MessageBoxButtons.OK, MessageBoxIcon.Error)
        End Try
    End Sub

    Private Sub buttonDial_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles buttonDial.Click
        ' Check if a line was selected
        Dim objLine As TapiLine
        objLine = comboBoxLine.SelectedItem
        If objLine Is Nothing Then
            MessageBox.Show("Please select a line first.")
            Return
        End If
        ' Check if a number was entered
        Dim number As String
        number = textBoxNumber.Text
        If number Is Nothing Or number.Length = 0 Then
            MessageBox.Show("Please enter a number to call.")
            Return
        End If
        Try
            ' Check if this line supports voice functions (playing wav files and speech)
            If Not objLine.SupportsVoice Then
                MessageBox.Show("This line does not support audio playback/recording and cannot be used to play wav files.\n" + _
                                "If this is a voice modem, make sure you have installed TAPI-compatible voice driver for the modem.\n\n" + _
                                "Outgoing call will be placed in interactive mode and terminated 30 seconds after the call is connected.")
            End If
            ' Open selected line for outgoing calls if it wasn't already open
            If Not objLine.IsOpen Then
                objLine.Open(False, AddressOf CallHandler)
            End If
            objLine.SpeechVoice = comboBoxVoice.SelectedItem
            ' Set no answer timeout to 30 seconds (approx. 5 rings)
            objLine.NoAnswerTimeout = 30
            ' Dial entered number without the dialing rules
            objLine.Dial(number, False)
            Dim msg As String
            msg = String.Format("Dialing {0} on line '{1}'", number, objLine.Name)
            AddToLog(msg)
        Catch exc As TapiException
            MessageBox.Show(exc.Message, "TapiException!", MessageBoxButtons.OK, MessageBoxIcon.Error)
        Catch exc As Exception
            MessageBox.Show(exc.Message, "Exception!", MessageBoxButtons.OK, MessageBoxIcon.Error)
        End Try
    End Sub

    ' This method will be called when outgoing call is connected.
    ' All call flow processing is performed in this method.
    ' Please note that voice modems don't support answer detection and
    ' this method will be called before the call is answered when the phone is still ringing.
    ' This method is called on a separate thread and is executed in the background.
    ' Several copies of this method can be executed at the same time (one for each line).
    ' If you call methods of your class, make sure those methods are thread-safe.
    ' WinForms controls can't be accessed directly in this method.
    Private Sub CallHandler(ByVal objCall As TapiCall)
        Try
            Dim msg As String
            msg = String.Format("Connected call to {0}.", objCall.CalledID)
            ' Because it's another thread, we have to use Invoke to access UI controls
            listBoxLog.Invoke(m_addToLogDelegate, msg)

            ' If the line doesn't support voice, wait 30 seconds then terminate the call
            If Not objCall.Line.SupportsVoice Then
                System.Threading.Thread.Sleep(30000)
                Return
            End If

            ' Try to deliver a message, up to 5 times.
            For retry As Integer = 1 To 5
                ' Play a message:
                ' "Hello! This is a message from Traysoft telephony demo."
                ' "Please press 1 to listen to the message."
                objCall.Play("..\..\Wavs\hello.wav")
                objCall.Play("..\..\Wavs\press1.wav")
                ' Wait for a digit (up to 10 seconds)
                Dim digit As String
                digit = objCall.WaitForDigit(10)
                If digit = "1" Then
                    ' Button 1 was pressed, speak using selected voice
                    If Not objCall.Line.SpeechVoice Is Nothing Then
                        msg = String.Format("You have selected {0} voice.", _
                            objCall.Line.SpeechVoice.Name)
                    Else
                        msg = "You have selected default computer voice."
                    End If
                    objCall.Speak(msg, True)
                    ' Wait until speaking is finished
                    objCall.WaitUntilDone(10)
                    msg = String.Format("Message delivered to {0}.", objCall.CalledID)
                    ' Because it's another thread, we have to use Invoke to access UI controls
                    listBoxLog.Invoke(m_addToLogDelegate, msg)
                    Exit For
                End If
            Next

            ' Play goodbye message:
            ' "Thank you for using Traysoft telephony demo. Goodbye, and have a good day."
            objCall.Play("..\..\Wavs\goodbye.wav", True)
            ' Wait until playing is finished
            objCall.WaitUntilDone(10)
        Catch exc As TapiDisconnectException
            ' This exception is thrown if the other party hung up before we called Disconnect().
            listBoxLog.Invoke(m_addToLogDelegate, "The called party hung up.")
        Catch exc As Exception
            Dim msg As String
            msg = String.Format("Exception in CallHandler! {0}", exc.Message)
            listBoxLog.Invoke(m_addToLogDelegate, msg)
        Finally
            ' Disconnect the call to release the line
            objCall.Disconnect()
        End Try
    End Sub

    ' This event handler is called when the call has been disconnected.
    ' After the current call is disconnected, another call can be placed on the line.
    Private Sub OnCallDisconnected(ByVal sender As Object, ByVal args As TapiEventArgs)
        Dim msg As String
        msg = String.Format("Disconnected call to {0} on line '{1}'", _
                       args.Call.CalledID, args.Line.Name)
        ' Check why the call was disconnected
        Select Case args.Call.DisconnectMode
            Case TapiDisconnectMode.Busy
                msg += ", the number was busy"
            Case TapiDisconnectMode.NoAnswer
                msg += ", there was no answer"
            Case TapiDisconnectMode.NoDialTone
                msg += ", there was no dialtone"
            Case TapiDisconnectMode.Error
                msg += ", an error occurred"
        End Select
        AddToLog(msg)
    End Sub

    ' This event handler is called when AddTapi error occurs asynchronously
    Private Sub OnTapiError(ByVal sender As Object, ByVal args As TapiErrorEventArgs)
        ' Display error in the log
        Dim msg As String
        If Not args.Line Is Nothing Then
            msg = String.Format("TapiError event, line '{0}'. {1}", _
                    args.Line.Name, args.Message)
        Else
            msg = String.Format("TapiError event. {0}", args.Message)
        End If
        AddToLog(msg)
    End Sub

    ' This event handler is called when new line has been added
    ' while application is running, for example USB modem was connected.
    Private Sub OnLineAdded(ByVal sender As Object, ByVal args As TapiEventArgs)
        UpdateLinesCombobox()
    End Sub

    ' This event handler is called when the line was forcibly closed
    ' by Windows because of hardware error or configuration change.
    Private Sub OnLineClosed(ByVal sender As Object, ByVal args As TapiEventArgs)
        ' Display message in the log
        Dim msg As String
        msg = String.Format("LineClosed event, line '{0}' was forcibly closed by Windows.", _
                                args.Line.Name)
        AddToLog(msg)
        UpdateLinesCombobox()
    End Sub

    ' This event handler is called when new line has been removed
    ' while application is running, for example USB modem was disconnected.
    Public Sub OnLineRemoved(ByVal sender As Object, ByVal args As TapiEventArgs)
        UpdateLinesCombobox()
    End Sub

    Private Sub AddToLog(ByVal msg As String)
        Dim ind As Integer
        ind = listBoxLog.Items.Add(msg)
        ' Ensure that the added item is visible
        listBoxLog.TopIndex = ind
    End Sub

    Private Sub UpdateLinesCombobox()
        comboBoxLine.Items.Clear()
        For Each objLine As TapiLine In TapiApp.Lines
            comboBoxLine.Items.Add(objLine)
        Next
    End Sub

End Class
