Option Explicit On

Imports Traysoft.AddTapi

Public Class Form1

    Private Sub Form1_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        Try
            ' Set the serial number if you purchased AddTapi.NET
            'TapiApp.SerialNumber = "ABCD-EFGH-IJKL"
            ' Initialize AddTapi
            TapiApp.Initialize("CallerID")
            ' Subscribe to events from AddTapi
            AddHandler TapiApp.TapiError, AddressOf OnTapiError
            AddHandler TapiApp.LineAdded, AddressOf OnLineAdded
            AddHandler TapiApp.LineClosed, AddressOf OnLineClosed
            AddHandler TapiApp.LineRemoved, AddressOf OnLineRemoved
            AddHandler TapiApp.IncomingCall, AddressOf OnIncomingCall
            AddHandler TapiApp.OutgoingCall, AddressOf OnOutgoingCall
            AddHandler TapiApp.CallConnected, AddressOf OnCallConnected
            AddHandler TapiApp.CallDisconnected, AddressOf OnCallDisconnected
            ' Show line names
            UpdateLinesCombobox()
        Catch exc As TapiException
            MessageBox.Show(exc.Message, "TapiException!", MessageBoxButtons.OK, MessageBoxIcon.Error)
            Me.Close()
        Catch exc As Exception
            MessageBox.Show(exc.Message, "Exception!", MessageBoxButtons.OK, MessageBoxIcon.Error)
            Me.Close()
        End Try
    End Sub

    Private Sub Form1_FormClosed(ByVal sender As System.Object, ByVal e As System.Windows.Forms.FormClosedEventArgs) Handles MyBase.FormClosed
        Try
            TapiApp.Shutdown()
        Catch exc As TapiException
            MessageBox.Show(exc.Message, "TapiException!", MessageBoxButtons.OK, MessageBoxIcon.Error)
        Catch exc As Exception
            MessageBox.Show(exc.Message, "Exception!", MessageBoxButtons.OK, MessageBoxIcon.Error)
        End Try
    End Sub

    Private Sub buttonOpen_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles buttonOpen.Click
        ' Check if a line was selected
        Dim objLine As TapiLine
        objLine = comboBoxLine.SelectedItem
        If objLine Is Nothing Then
            MessageBox.Show("Please select a line first.")
            Return
        End If
        ' Check if the line is already open
        If objLine.IsOpen Then
            AddToLog("Already monitoring line " + objLine.Name)
            Return
        End If
        ' Open selected line
        Try
            objLine.RingsToAnswer = 0
            objLine.Open(True, Nothing)
            AddToLog("Started monitoring line " + objLine.Name)
        Catch exc As TapiException
            MessageBox.Show(exc.Message, "TapiException!", MessageBoxButtons.OK, MessageBoxIcon.Error)
        Catch exc As Exception
            MessageBox.Show(exc.Message, "Exception!", MessageBoxButtons.OK, MessageBoxIcon.Error)
        End Try
    End Sub

    Private Sub buttonClose_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles buttonClose.Click
        ' Check if a line was selected
        Dim objLine As TapiLine
        objLine = comboBoxLine.SelectedItem
        If objLine Is Nothing Then
            MessageBox.Show("Please select a line first.")
            Return
        End If
        ' Close selected line
        Try
            If objLine.IsOpen Then
                objLine.Close()
            End If
            AddToLog("Stopped monitoring line " + objLine.Name)
        Catch exc As TapiException
            MessageBox.Show(exc.Message, "TapiException!", MessageBoxButtons.OK, MessageBoxIcon.Error)
        Catch exc As Exception
            MessageBox.Show(exc.Message, "Exception!", MessageBoxButtons.OK, MessageBoxIcon.Error)
        End Try
    End Sub

    ' This event handler is called when a new incoming call is received.
    Private Sub OnIncomingCall(ByVal sender As Object, ByVal args As TapiEventArgs)
        ' Display message in the log
        Dim msg As String
        msg = String.Format("Incoming call from {0} {1} on line '{2}'.", _
                       args.Call.CallerIDName, args.Call.CallerID, args.Line.Name)
        AddToLog(msg)
    End Sub

    ' This event handler is called when a new outgoing call is detected.
    Private Sub OnOutgoingCall(ByVal sender As Object, ByVal args As TapiEventArgs)
        ' Display message in the log
        Dim msg As String
        msg = String.Format("Outgoing call to {0} on line '{1}'.", _
                       args.Call.CalledID, args.Line.Name)
        AddToLog(msg)
    End Sub

    ' This event handler is called when the call has been connected.
    Private Sub OnCallConnected(ByVal sender As Object, ByVal args As TapiEventArgs)
        Dim msg As String
        If args.Call.Direction = TapiCallDirection.Incoming Then
            msg = String.Format("Connected incoming call from {0} on line '{1}'", _
                           args.Call.CallerID, args.Line.Name)
        Else
            msg = String.Format("Connected outgoing call to {0} on line '{1}'", _
                           args.Call.CalledID, args.Line.Name)
        End If
        AddToLog(msg)
    End Sub

    ' This event handler is called when the call has been disconnected.
    Private Sub OnCallDisconnected(ByVal sender As Object, ByVal args As TapiEventArgs)
        Dim msg As String
        msg = String.Format("Disconnected the call on line '{0}'", args.Line.Name)
        AddToLog(msg)
    End Sub

    ' This event handler is called when AddTapi error occurs asynchronously
    Private Sub OnTapiError(ByVal sender As Object, ByVal args As TapiErrorEventArgs)
        ' Display error in the log
        Dim msg As String
        If Not args.Line Is Nothing Then
            msg = String.Format("TapiError event, line '{0}'. {1}", _
                    args.Line.Name, args.Message)
        Else
            msg = String.Format("TapiError event. {0}", args.Message)
        End If
        AddToLog(msg)
    End Sub

    ' This event handler is called when new line has been added
    ' while application is running, for example USB modem was connected.
    Private Sub OnLineAdded(ByVal sender As Object, ByVal args As TapiEventArgs)
        UpdateLinesCombobox()
    End Sub

    ' This event handler is called when the line was forcibly closed
    ' by Windows because of hardware error or configuration change.
    Private Sub OnLineClosed(ByVal sender As Object, ByVal args As TapiEventArgs)
        ' Display message in the log
        Dim msg As String
        msg = String.Format("LineClosed event, line '{0}' was forcibly closed by Windows.", _
                                args.Line.Name)
        AddToLog(msg)
        UpdateLinesCombobox()
    End Sub

    ' This event handler is called when new line has been removed
    ' while application is running, for example USB modem was disconnected.
    Public Sub OnLineRemoved(ByVal sender As Object, ByVal args As TapiEventArgs)
        UpdateLinesCombobox()
    End Sub

    Private Sub AddToLog(ByVal msg As String)
        Dim ind As Integer
        ind = listBoxLog.Items.Add(msg)
        ' Ensure that the added item is visible
        listBoxLog.TopIndex = ind
    End Sub

    Private Sub UpdateLinesCombobox()
        comboBoxLine.Items.Clear()
        For Each objLine As TapiLine In TapiApp.Lines
            comboBoxLine.Items.Add(objLine)
        Next
    End Sub

End Class
