﻿Option Explicit On

Imports Traysoft.AddTapi

Public Class Form1

    Private Sub Form1_Load(sender As System.Object, e As System.EventArgs) Handles MyBase.Load
        Try
            ' Set the serial number if you purchased AddTapi.NET
            'TapiApp.SerialNumber = "ABCD-EFGH-IJKL"
            ' Initialize AddTapi
            TapiApp.Initialize("Dial")
            ' Subscribe to events from AddTapi
            AddHandler TapiApp.TapiError, AddressOf OnTapiError
            AddHandler TapiApp.LineAdded, AddressOf OnLineAdded
            AddHandler TapiApp.LineClosed, AddressOf OnLineClosed
            AddHandler TapiApp.LineRemoved, AddressOf OnLineRemoved
            AddHandler TapiApp.CallConnected, AddressOf OnCallConnected
            AddHandler TapiApp.CallDisconnected, AddressOf OnCallDisconnected
            ' Show line names in the combobox
            UpdateLinesCombobox()
        Catch exc As TapiException
            MessageBox.Show(exc.Message, "TapiException!", MessageBoxButtons.OK, MessageBoxIcon.Error)
            Me.Close()
        Catch exc As Exception
            MessageBox.Show(exc.Message, "Exception!", MessageBoxButtons.OK, MessageBoxIcon.Error)
            Me.Close()
        End Try
    End Sub


    Private Sub Form1_FormClosed(sender As System.Object, e As System.Windows.Forms.FormClosedEventArgs) Handles MyBase.FormClosed
        Try
            TapiApp.Shutdown()
        Catch exc As TapiException
            MessageBox.Show(exc.Message, "TapiException!", MessageBoxButtons.OK, MessageBoxIcon.Error)
        Catch exc As Exception
            MessageBox.Show(exc.Message, "Exception!", MessageBoxButtons.OK, MessageBoxIcon.Error)
        End Try
    End Sub

    Private Sub buttonDial_Click(sender As System.Object, e As System.EventArgs) Handles buttonDial.Click
        ' Check if a line was selected
        Dim objLine As TapiLine
        objLine = comboBoxLine.SelectedItem
        If objLine Is Nothing Then
            MessageBox.Show("Please select a line first.")
            Return
        End If
        ' Check if a number was entered
        Dim number As String
        number = textBoxNumber.Text
        If number Is Nothing Or number.Length = 0 Then
            MessageBox.Show("Please enter a number to call.")
            Return
        End If
        Try
            ' Open selected line for outgoing calls if it wasn't already open
            If Not objLine.IsOpen Then
                objLine.Open(False, Nothing, TapiMediaMode.InteractiveVoice)
            End If
            ' Dial entered number without the dialing rules
            objLine.Dial(number, False)
            Dim msg As String
            msg = String.Format("Dialing {0} on line '{1}'", number, objLine.Name)
            AddToLog(msg)
        Catch exc As TapiException
            MessageBox.Show(exc.Message, "TapiException!", MessageBoxButtons.OK, MessageBoxIcon.Error)
        Catch exc As Exception
            MessageBox.Show(exc.Message, "Exception!", MessageBoxButtons.OK, MessageBoxIcon.Error)
        End Try
    End Sub

    Private Sub buttonDisconnect_Click(sender As System.Object, e As System.EventArgs) Handles buttonDisconnect.Click
        ' Check if a line was selected
        Dim objLine As TapiLine
        objLine = comboBoxLine.SelectedItem
        If objLine Is Nothing Then
            Return
        End If
        Try
            ' Find a call that has not been disconnected and disconnect it
            For Each objCall As TapiCall In objLine.Calls
                If objCall.State <> TapiCallState.Idle And objCall.State <> TapiCallState.Disconnected Then
                    Dim msg As String
                    msg = String.Format("Disconnecting call to {0} on line '{1}'", objCall.CalledID, objLine.Name)
                    AddToLog(msg)
                    objCall.Disconnect()
                End If
            Next
        Catch exc As TapiException
            MessageBox.Show(exc.Message, "TapiException!", MessageBoxButtons.OK, MessageBoxIcon.Error)
        Catch exc As Exception
            MessageBox.Show(exc.Message, "Exception!", MessageBoxButtons.OK, MessageBoxIcon.Error)
        End Try
    End Sub

    ' This event handler is called when the call has been connected.
    Private Sub OnCallConnected(ByVal sender As Object, ByVal args As TapiEventArgs)
        Dim msg As String
        msg = String.Format("Connected call to {0} on line '{1}', pick up the handset to talk", _
                       args.Call.CalledID, args.Line.Name)
        AddToLog(msg)
    End Sub

    ' This event handler is called when the call has been disconnected.
    ' After the current call is disconnected, another call can be placed on the line.
    Private Sub OnCallDisconnected(ByVal sender As Object, ByVal args As TapiEventArgs)
        Dim msg As String
        msg = String.Format("Disconnected call to {0} on line '{1}'", _
                       args.Call.CalledID, args.Line.Name)
        ' Check why the call was disconnected
        Select Case args.Call.DisconnectMode
            Case TapiDisconnectMode.Busy
                msg += ", the number was busy"
            Case TapiDisconnectMode.NoAnswer
                msg += ", there was no answer"
            Case TapiDisconnectMode.NoDialTone
                msg += ", there was no dialtone"
            Case TapiDisconnectMode.Error
                msg += ", an error occurred"
        End Select
        AddToLog(msg)

        ' Use TapiCall.Disconnect to release the line
        Try
            args.Call.Disconnect()
        Catch exc As TapiException
            MessageBox.Show(exc.Message, "TapiException!", MessageBoxButtons.OK, MessageBoxIcon.Error)
        Catch exc As Exception
            MessageBox.Show(exc.Message, "Exception!", MessageBoxButtons.OK, MessageBoxIcon.Error)
        End Try
    End Sub

    ' This event handler is called when AddTapi error occurs asynchronously
    Private Sub OnTapiError(ByVal sender As Object, ByVal args As TapiErrorEventArgs)
        ' Display error in the log
        Dim msg As String
        If Not args.Line Is Nothing Then
            msg = String.Format("TapiError event, line '{0}'. {1}", _
                    args.Line.Name, args.Message)
        Else
            msg = String.Format("TapiError event. {0}", args.Message)
        End If
        AddToLog(msg)
    End Sub

    ' This event handler is called when new line has been added
    ' while application is running, for example USB modem was connected.
    Private Sub OnLineAdded(ByVal sender As Object, ByVal args As TapiEventArgs)
        UpdateLinesCombobox()
    End Sub

    ' This event handler is called when the line was forcibly closed
    ' by Windows because of hardware error or configuration change.
    Private Sub OnLineClosed(ByVal sender As Object, ByVal args As TapiEventArgs)
        ' Display message in the log
        Dim msg As String
        msg = String.Format("LineClosed event, line '{0}' was forcibly closed by Windows.", _
                                args.Line.Name)
        AddToLog(msg)
        UpdateLinesCombobox()
    End Sub

    ' This event handler is called when new line has been removed
    ' while application is running, for example USB modem was disconnected.
    Public Sub OnLineRemoved(ByVal sender As Object, ByVal args As TapiEventArgs)
        UpdateLinesCombobox()
    End Sub

    Private Sub AddToLog(ByVal msg As String)
        Dim ind As Integer
        ind = listBoxLog.Items.Add(msg)
        ' Ensure that the added item is visible
        listBoxLog.TopIndex = ind
    End Sub

    Private Sub UpdateLinesCombobox()
        comboBoxLine.Items.Clear()
        For Each objLine As TapiLine In TapiApp.Lines
            comboBoxLine.Items.Add(objLine)
        Next
    End Sub

End Class
