Option Explicit On

Imports Traysoft.AddTapi

Public Class Form1

    Delegate Sub AddListItem(ByVal item As String)
    Dim m_addToLogDelegate As AddListItem

    Private Sub Form1_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        m_addToLogDelegate = New AddListItem(AddressOf AddToLog)
        Try
            ' Set the serial number if you purchased AddTapi.NET
            'TapiApp.SerialNumber = "ABCD-EFGH-IJKL"
            ' Initialize AddTapi
            TapiApp.Initialize("IVR Sample")
            ' Subscribe to events from AddTapi
            AddHandler TapiApp.TapiError, AddressOf OnTapiError
            AddHandler TapiApp.LineAdded, AddressOf OnLineAdded
            AddHandler TapiApp.LineClosed, AddressOf OnLineClosed
            AddHandler TapiApp.LineRemoved, AddressOf OnLineRemoved
            AddHandler TapiApp.IncomingCall, AddressOf OnIncomingCall
            ' Fill out voice combobox with a list of available text-to-speech voices
            For Each voice As TapiVoice In TapiApp.Voices
                comboBoxVoice.Items.Add(voice)
            Next
            ' Update controls to show line names
            UpdateFormControls()
        Catch exc As TapiException
            MessageBox.Show(exc.Message, "TapiException!", MessageBoxButtons.OK, MessageBoxIcon.Error)
            Me.Close()
        Catch exc As Exception
            MessageBox.Show(exc.Message, "Exception!", MessageBoxButtons.OK, MessageBoxIcon.Error)
            Me.Close()
        End Try
    End Sub

    Private Sub Form1_FormClosed(ByVal sender As System.Object, ByVal e As System.Windows.Forms.FormClosedEventArgs) Handles MyBase.FormClosed
        Try
            TapiApp.Shutdown()
        Catch exc As TapiException
            MessageBox.Show(exc.Message, "TapiException!", MessageBoxButtons.OK, MessageBoxIcon.Error)
        Catch exc As Exception
            MessageBox.Show(exc.Message, "Exception!", MessageBoxButtons.OK, MessageBoxIcon.Error)
        End Try
    End Sub

    ' This method will be called when incoming call is answered and connected.
    ' All call flow processing is performed in this method.
    ' This method is called on a separate thread and is executed in the background.
    ' Several copies of this method can be executed at the same time (one for each line).
    ' If you call methods of your class, make sure those methods are thread-safe.
    ' WinForms controls can't be accessed directly in this method.
    Private Sub CallHandler(ByVal objCall As TapiCall)
        Try
            Dim msg As String
            msg = String.Format("Answered call from {0}.", objCall.CallerID)
            ' Because it's another thread, we have to use Invoke to access UI controls
            listBoxLog.Invoke(m_addToLogDelegate, msg)

            ' Play a welcome message:
            ' "Welcome to Traysoft telephony demo!"
            objCall.Play("..\..\Wavs\welcome.wav")
            ' Wait until playback is completed (10 seconds maximum).
            objCall.WaitUntilDone(10)
            ' Ask for a pin:
            ' "Please enter your 4-digit pin. In this demo you can enter any 4 digits."
            objCall.Play("..\..\Wavs\askpin.wav")
            Dim pin As String
            pin = objCall.GatherDigits(4, Nothing, 20, 5)
            If (pin Is Nothing) Or (pin.Length < 4) Then
                ' Pin wasn't entered, play a message and finish the call
                ' "You did not enter the pin."
                objCall.Play("..\..\Wavs\nopin.wav")
                ' Wait until playing is finished
                objCall.WaitUntilDone(10)
                Return
            End If
            ' Show pin in the log
            msg = String.Format("Entered pin: {0}", pin)
            listBoxLog.Invoke(m_addToLogDelegate, msg)

            ' Play main menu prompt
            Dim menuSelection As String
            Do
                ' Play menu prompt:
                ' "Press 1 to hear selected text-to-speech voice."
                ' "Press 2 to hear generated DTMF digits."
                ' "Press star to disconnect."
                objCall.Play("..\..\Wavs\menu1.wav")
                objCall.Play("..\..\Wavs\menu2.wav")
                objCall.Play("..\..\Wavs\menu3.wav")
                ' Wait for a digit (up to 20 seconds)
                menuSelection = objCall.WaitForDigit(20)
                If Not menuSelection Is Nothing Then
                    ' Process selected menu option
                    Select Case menuSelection
                        Case "1"
                            ' Option 1 was chosen, speak using selected voice
                            If Not objCall.Line.SpeechVoice Is Nothing Then
                                msg = String.Format("You have selected {0} voice.", objCall.Line.SpeechVoice.Name)
                            Else
                                msg = "You have selected default computer voice."
                            End If
                            objCall.Speak(msg, True)
                        Case "2"
                            ' Option 2 was chosen
                            objCall.Speak("Digits 1 2 3 4", True)
                            objCall.WaitUntilDone(10)
                            ' Send DTMF digits to the caller
                            objCall.GenerateDigits("1234")
                            ' Wait 3 seconds before we continue
                            System.Threading.Thread.Sleep(3000)
                    End Select
                End If
            Loop Until menuSelection = "*"

            ' Play goodbye message:
            ' "Thank you for using Traysoft telephony demo. Goodbye, and have a good day."
            objCall.Play("..\..\Wavs\goodbye.wav", True)
            ' Wait until playing is finished
            objCall.WaitUntilDone(10)
        Catch exc As TapiDisconnectException
            ' This exception is thrown if the caller hung up before we called Disconnect().
            listBoxLog.Invoke(m_addToLogDelegate, "The caller hung up.")
        Catch exc As Exception
            Dim msg As String
            msg = String.Format("Exception in CallHandler! {0}", exc.Message)
            listBoxLog.Invoke(m_addToLogDelegate, msg)
        Finally
            ' Disconnect the call to clear up the line
            objCall.Disconnect()
        End Try
    End Sub

    Private Sub OpenLine(ByVal lineIndex As Integer)
        Try
            ' If specified line doesn't exist, don't do anything
            If lineIndex >= TapiApp.Lines.Count Then
                Return
            End If
            ' Open specified line for incoming calls
            Dim objLine As TapiLine
            objLine = TapiApp.Lines(lineIndex)
            ' Check if this line supports voice functions (playing wav files and speech)
            If Not objLine.SupportsVoice Then
                MessageBox.Show("This line does not support audio playback/recording and cannot be used to play messages to the callers. " + _
                            "If this is a voice modem, make sure you have installed TAPI-compatible voice driver for the modem.")
                Return
            End If
            objLine.Open(True, AddressOf CallHandler)
            objLine.RingsToAnswer = numericUpDownRings.Value
            UpdateFormControls()
        Catch exc As TapiException
            MessageBox.Show(exc.Message, "TapiException!", MessageBoxButtons.OK, MessageBoxIcon.Error)
        Catch exc As Exception
            MessageBox.Show(exc.Message, "Exception!", MessageBoxButtons.OK, MessageBoxIcon.Error)
        End Try
    End Sub

    Private Sub CloseLine(ByVal lineIndex As Integer)
        Try
            ' If specified line doesn't exist, don't do anything
            If lineIndex >= TapiApp.Lines.Count Then
                Return
            End If
            ' Close specified line
            Dim objLine As TapiLine
            objLine = TapiApp.Lines(lineIndex)
            objLine.Close()
            UpdateFormControls()
        Catch exc As TapiException
            MessageBox.Show(exc.Message, "TapiException!", MessageBoxButtons.OK, MessageBoxIcon.Error)
        Catch exc As Exception
            MessageBox.Show(exc.Message, "Exception!", MessageBoxButtons.OK, MessageBoxIcon.Error)
        End Try
    End Sub

    ' This event handler is called when AddTapi error occurs asynchronously
    Private Sub OnTapiError(ByVal sender As Object, ByVal args As TapiErrorEventArgs)
        ' Display error in the log
        Dim msg As String
        If Not args.Line Is Nothing Then
            msg = String.Format("TapiError event, line '{0}'. {1}", _
                    args.Line.Name, args.Message)
        Else
            msg = String.Format("TapiError event. {0}", args.Message)
        End If
        AddToLog(msg)
    End Sub

    ' This event handler is called when new line has been added
    ' while application is running, for example USB modem was connected.
    Private Sub OnLineAdded(ByVal sender As Object, ByVal args As TapiEventArgs)
        UpdateFormControls()
    End Sub

    ' This event handler is called when the line was forcibly closed
    ' by Windows because of hardware error or configuration change.
    Private Sub OnLineClosed(ByVal sender As Object, ByVal args As TapiEventArgs)
        ' Display message in the log
        Dim msg As String
        msg = String.Format("LineClosed event, line '{0}' was forcibly closed by Windows.", args.Line.Name)
        AddToLog(msg)
        UpdateFormControls()
    End Sub

    ' This event handler is called when new line has been removed
    ' while application is running, for example USB modem was disconnected.
    Private Sub OnLineRemoved(ByVal sender As Object, ByVal args As TapiEventArgs)
        UpdateFormControls()
    End Sub

    ' This event handler is called when new call is received.
    ' You can use this handler for logging, UI updates etc.
    Private Sub OnIncomingCall(ByVal sender As Object, ByVal args As TapiEventArgs)
        ' Display message in the log
        Dim msg As String
        msg = String.Format("Incoming call from {0} on line '{1}'.", _
                       args.Call.CallerID, args.Line.Name)
        AddToLog(msg)
    End Sub

    ' This method displays line names and enables/disables Open and Close buttons
    Private Sub UpdateFormControls()
        textBoxLine1.Text = ""
        buttonLine1Open.Enabled = False
        buttonLine1Close.Enabled = False
        If TapiApp.Lines.Count >= 1 Then
            ' Line 1 is present
            Dim line As TapiLine
            line = TapiApp.Lines(0)
            textBoxLine1.Text = line.Name
            If line.IsOpen Then
                buttonLine1Close.Enabled = True
            Else
                buttonLine1Open.Enabled = True
            End If
        End If

        textBoxLine2.Text = ""
        buttonLine2Open.Enabled = False
        buttonLine2Close.Enabled = False
        If TapiApp.Lines.Count >= 2 Then
            ' Line 2 is present
            Dim line As TapiLine
            line = TapiApp.Lines(1)
            textBoxLine2.Text = line.Name
            If line.IsOpen Then
                buttonLine2Close.Enabled = True
            Else
                buttonLine2Open.Enabled = True
            End If
        End If

        textBoxLine3.Text = ""
        buttonLine3Open.Enabled = False
        buttonLine3Close.Enabled = False
        If TapiApp.Lines.Count >= 3 Then
            ' Line 3 is present
            Dim line As TapiLine
            line = TapiApp.Lines(2)
            textBoxLine3.Text = line.Name
            If line.IsOpen Then
                buttonLine3Close.Enabled = True
            Else
                buttonLine3Open.Enabled = True
            End If
        End If

        textBoxLine4.Text = ""
        buttonLine4Open.Enabled = False
        buttonLine4Close.Enabled = False
        If TapiApp.Lines.Count >= 4 Then
            ' Line 4 is present
            Dim line As TapiLine
            line = TapiApp.Lines(3)
            textBoxLine4.Text = line.Name
            If line.IsOpen Then
                buttonLine4Close.Enabled = True
            Else
                buttonLine4Open.Enabled = True
            End If
        End If

        textBoxLine5.Text = ""
        buttonLine5Open.Enabled = False
        buttonLine5Close.Enabled = False
        If TapiApp.Lines.Count >= 5 Then
            ' Line 5 is present
            Dim line As TapiLine
            line = TapiApp.Lines(4)
            textBoxLine5.Text = line.Name
            If line.IsOpen Then
                buttonLine5Close.Enabled = True
            Else
                buttonLine5Open.Enabled = True
            End If
        End If
    End Sub

    Private Sub comboBoxVoice_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles comboBoxVoice.SelectedIndexChanged
        ' Selected text-to-speech voice has changed, assign new voice to all lines
        For Each line As TapiLine In TapiApp.Lines
            line.SpeechVoice = comboBoxVoice.SelectedItem
        Next
    End Sub

    Private Sub numericUpDownRings_ValueChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles numericUpDownRings.ValueChanged
        ' Number of rings to answer has changed, assign new value to all lines
        For Each line As TapiLine In TapiApp.Lines
            line.RingsToAnswer = numericUpDownRings.Value
        Next
    End Sub

    Private Sub AddToLog(ByVal msg As String)
        Dim ind As Integer
        ind = listBoxLog.Items.Add(msg)
        ' Ensure that the added item is visible
        listBoxLog.TopIndex = ind
    End Sub

    Private Sub buttonLine1Open_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles buttonLine1Open.Click
        OpenLine(0)
    End Sub

    Private Sub buttonLine2Open_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles buttonLine2Open.Click
        OpenLine(1)
    End Sub

    Private Sub buttonLine3Open_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles buttonLine3Open.Click
        OpenLine(2)
    End Sub

    Private Sub buttonLine4Open_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles buttonLine4Open.Click
        OpenLine(3)
    End Sub

    Private Sub buttonLine5Open_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles buttonLine5Open.Click
        OpenLine(4)
    End Sub

    Private Sub buttonLine1Close_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles buttonLine1Close.Click
        CloseLine(0)
    End Sub

    Private Sub buttonLine2Close_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles buttonLine2Close.Click
        CloseLine(1)
    End Sub

    Private Sub buttonLine3Close_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles buttonLine3Close.Click
        CloseLine(2)
    End Sub

    Private Sub buttonLine4Close_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles buttonLine4Close.Click
        CloseLine(3)
    End Sub

    Private Sub buttonLine5Close_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles buttonLine5Close.Click
        CloseLine(4)
    End Sub

End Class
